@@ -2,7 +2,8 @@
 /* vim: set expandtab sw=4 ts=4 sts=4: */
 /**
  *
- * @version $Id: db_structure.php 11244 2008-05-08 01:30:22Z lem9 $
+ * @version $Id: db_structure.php 13034 2009-10-12 21:47:40Z lem9 $
+ * @package phpMyAdmin
  */
 
 /**
@@ -11,6 +12,8 @@
 require_once './libraries/common.inc.php';
 require_once './libraries/Table.class.php';
 
+$GLOBALS['js_include'][] = 'mootools.js';
+
 /**
  * Prepares the tables list if the user where not redirected to this script
  * because there is no table in the database ($is_info is true)
@@ -22,7 +25,9 @@ if (empty($is_info)) {
         $action = 'db_structure.php';
         $err_url = 'db_structure.php?'. PMA_generate_common_url($db);
         require './libraries/mult_submits.inc.php';
-        $message = $strSuccess;
+        if (empty($message)) {
+            $message = PMA_Message::success();
+        }
     }
     require './libraries/db_common.inc.php';
     $url_query .= '&amp;goto=db_structure.php';
@@ -52,10 +57,8 @@ if ($num_tables == 0) {
 
 require_once './libraries/bookmark.lib.php';
 
-if (PMA_MYSQL_INT_VERSION >= 40101) {
-    require_once './libraries/mysql_charsets.lib.php';
-    $db_collation = PMA_getDbCollation($db);
-}
+require_once './libraries/mysql_charsets.lib.php';
+$db_collation = PMA_getDbCollation($db);
 
 // Display function
 /**
@@ -63,7 +66,6 @@ if (PMA_MYSQL_INT_VERSION >= 40101) {
  * display table header (<table><thead>...</thead><tbody>)
  *
  * @uses    PMA_showHint()
- * @uses    PMA_MYSQL_INT_VERSION
  * @uses    $GLOBALS['cfg']['PropertiesNumColumns']
  * @uses    $GLOBALS['is_show_stats']
  * @uses    $GLOBALS['strTable']
@@ -75,6 +77,7 @@ if (PMA_MYSQL_INT_VERSION >= 40101) {
  * @uses    $GLOBALS['strSize']
  * @uses    $GLOBALS['strOverhead']
  * @uses    $GLOBALS['structure_tbl_col_cnt']
+ * @uses    PMA_SortableTableHeader()
  * @param   boolean $db_is_information_schema
  */
 function PMA_TableHeader($db_is_information_schema = false)
@@ -89,25 +92,23 @@ function PMA_TableHeader($db_is_information_schema = false)
 
     echo '<table class="data" style="float: left;">' . "\n"
         .'<thead>' . "\n"
-        .'<tr><td></td>' . "\n"
-        .'    <th>' . $GLOBALS['strTable'] . '</th>' . "\n"
+        .'<tr><th></th>' . "\n"
+        .'    <th>' . PMA_SortableTableHeader($GLOBALS['strTable'], 'table') . '</th>' . "\n"
         .'    <th colspan="' . $action_colspan . '">' . "\n"
         .'        ' . $GLOBALS['strAction'] . "\n"
         .'    </th>'
-        .'    <th>' . $GLOBALS['strRecords']
-        .PMA_showHint($GLOBALS['strApproximateCount']) . "\n"
+        .'    <th>' . PMA_SortableTableHeader($GLOBALS['strRecords'], 'records')
+        .PMA_showHint(PMA_sanitize($GLOBALS['strApproximateCount'])) . "\n"
         .'    </th>' . "\n";
     if (!($GLOBALS['cfg']['PropertiesNumColumns'] > 1)) {
-        echo '    <th>' . $GLOBALS['strType'] . '</th>' . "\n";
+        echo '    <th>' . PMA_SortableTableHeader($GLOBALS['strType'], 'type') . '</th>' . "\n";
+        $cnt++;
+        echo '    <th>' . PMA_SortableTableHeader($GLOBALS['strCollation'], 'collation') . '</th>' . "\n";
         $cnt++;
-        if (PMA_MYSQL_INT_VERSION >= 40100) {
-            echo '    <th>' . $GLOBALS['strCollation'] . '</th>' . "\n";
-            $cnt++;
-        }
     }
     if ($GLOBALS['is_show_stats']) {
-        echo '    <th>' . $GLOBALS['strSize'] . '</th>' . "\n"
-           . '    <th>' . $GLOBALS['strOverhead'] . '</th>' . "\n";
+        echo '    <th>' . PMA_SortableTableHeader($GLOBALS['strSize'], 'size') . '</th>' . "\n"
+           . '    <th>' . PMA_SortableTableHeader($GLOBALS['strOverhead'], 'overhead') . '</th>' . "\n";
         $cnt += 2;
     }
     echo '</tr>' . "\n";
@@ -116,6 +117,65 @@ function PMA_TableHeader($db_is_information_schema = false)
     $GLOBALS['structure_tbl_col_cnt'] = $cnt + $action_colspan + 3;
 } // end function PMA_TableHeader()
 
+/**
+ * Creates a clickable column header for table information
+ *
+ * @param   string  title to use for the link
+ * @param   string  corresponds to sortable data name mapped in libraries/db_info.inc.php  
+ * @returns string  link to be displayed in the table header
+ */
+function PMA_SortableTableHeader($title, $sort)
+{
+    // Set some defaults
+    $requested_sort = 'table';
+    $requested_sort_order = 'ASC';
+    $future_sort_order = 'ASC';
+    
+    // If the user requested a sort
+    if (isset($_REQUEST['sort'])) {
+        $requested_sort = $_REQUEST['sort'];
+
+        if (isset($_REQUEST['sort_order'])) {
+            $requested_sort_order = $_REQUEST['sort_order'];
+        }
+    }
+
+    $order_img = '';
+    $order_link_params = array();
+    $order_link_params['title'] = $GLOBALS['strSort'];
+
+    // If this column was requested to be sorted.
+    if ($requested_sort == $sort) {
+        if ($requested_sort_order == 'ASC') {
+            $future_sort_order = 'DESC';
+            // current sort order is ASC
+            $order_img  = ' <img class="icon" src="' . $GLOBALS['pmaThemeImage'] . 's_asc.png" width="11" height="9" alt="'. $GLOBALS['strAscending'] . '" title="'. $GLOBALS['strAscending'] . '" id="sort_arrow" />';
+            // but on mouse over, show the reverse order (DESC)
+            $order_link_params['onmouseover'] = 'if(document.getElementById(\'sort_arrow\')){ document.getElementById(\'sort_arrow\').src=\'' . $GLOBALS['pmaThemeImage'] . 's_desc.png\'; }';
+            // on mouse out, show current sort order (ASC)
+            $order_link_params['onmouseout']  = 'if(document.getElementById(\'sort_arrow\')){ document.getElementById(\'sort_arrow\').src=\'' . $GLOBALS['pmaThemeImage'] . 's_asc.png\'; }';
+        } else {
+            // current sort order is DESC
+            $order_img  = ' <img class="icon" src="' . $GLOBALS['pmaThemeImage'] . 's_desc.png" width="11" height="9" alt="'. $GLOBALS['strDescending'] . '" title="'. $GLOBALS['strDescending'] . '" id="sort_arrow" />';
+            // but on mouse over, show the reverse order (ASC)
+            $order_link_params['onmouseover'] = 'if(document.getElementById(\'sort_arrow\')){ document.getElementById(\'sort_arrow\').src=\'' . $GLOBALS['pmaThemeImage'] . 's_asc.png\'; }';
+            // on mouse out, show current sort order (DESC)
+            $order_link_params['onmouseout']  = 'if(document.getElementById(\'sort_arrow\')){ document.getElementById(\'sort_arrow\').src=\'' . $GLOBALS['pmaThemeImage'] . 's_desc.png\'; }';
+        }
+    }
+
+    $_url_params = array(
+        'db' => $_REQUEST['db'],
+    );
+
+    $url = 'db_structure.php'.PMA_generate_common_url($_url_params);
+    // We set the position back to 0 every time they sort.
+    $url .= "&amp;pos=0&amp;sort=$sort&amp;sort_order=$future_sort_order";
+
+    return PMA_linkOrButton($url, $title . $order_img, $order_link_params);
+} // end function PMA_SortableTableHeader()
+
+
 $titles = array();
 if (true == $cfg['PropertiesIconic']) {
     $titles['Browse']     = '<img class="icon" width="16" height="16" src="' .$pmaThemeImage . 'b_browse.png" alt="' . $strBrowse . '" title="' . $strBrowse . '" />';
@@ -160,11 +220,19 @@ if (true == $cfg['PropertiesIconic']) {
 /**
  * Displays the tables list
  */
-
 $_url_params = array(
     'pos' => $pos,
     'db'  => $db);
 
+// Add the sort options if they exists
+if (isset($_REQUEST['sort'])) {
+    $_url_params['sort'] = $_REQUEST['sort'];
+}
+
+if (isset($_REQUEST['sort_order'])) {
+    $_url_params['sort_order'] = $_REQUEST['sort_order'];
+}
+
 PMA_listNavigator($total_num_tables, $pos, $_url_params, 'db_structure.php', 'frame_content', $GLOBALS['cfg']['MaxTableList']);
 
 ?>
@@ -185,22 +253,128 @@ $row_count      = 0;
 
 $hidden_fields = array();
 $odd_row       = true;
-$at_least_one_view_exceeds_max_count = false;
 $sum_row_count_pre = '';
 
+// added by rajk - for blobstreaming
+$PMA_Config = $_SESSION['PMA_Config'];
+
+if (!empty($PMA_Config))
+    $session_bs_tables = $PMA_Config->get('BLOBSTREAMING_TABLES'); // list of blobstreaming tables
+
+$tableReductionCount = 0;   // the amount to reduce the table count by
+
 foreach ($tables as $keyname => $each_table) {
-    if ($each_table['TABLE_ROWS'] === null || $each_table['TABLE_ROWS'] < $GLOBALS['cfg']['MaxExactCount']) {
-        $each_table['COUNTED'] = true;
-        $each_table['TABLE_ROWS'] = PMA_Table::countRecords($db,
-            $each_table['TABLE_NAME'], $return = true, $force_exact = true);
-    } else {
-        $each_table['COUNTED'] = false;
+    if (isset($session_bs_tables))
+    {
+        // compare table name against blobstreaming tables
+        foreach ($session_bs_tables as $table_key=>$table_val)
+            // if the table is a blobstreaming table, reduce table count and skip outer foreach loop
+            if ($table_key == $keyname)
+            {
+                $tableReductionCount++;
+                continue 2;
+            }
     }
 
+    // loic1: Patch from Joshua Nye <josh at boxcarmedia.com> to get valid
+    //        statistics whatever is the table type
+
+    $table_is_view = false;
     $table_encoded = urlencode($each_table['TABLE_NAME']);
-    // MySQL < 5.0.13 returns "view", >= 5.0.13 returns "VIEW"
-    $table_is_view = ($each_table['TABLE_TYPE'] === 'VIEW'
-                       || $each_table['TABLE_TYPE'] === 'SYSTEM VIEW');
+    // Sets parameters for links
+    $tbl_url_query = $url_query . '&amp;table=' . $table_encoded;
+    // do not list the previous table's size info for a view
+    $formatted_size = '-';
+    $unit = '';
+
+    switch ( $each_table['ENGINE']) {
+        // MyISAM, ISAM or Heap table: Row count, data size and index size
+        // are accurate.
+        case 'MyISAM' :
+        case 'ISAM' :
+        case 'HEAP' :
+        case 'MEMORY' :
+            if ($db_is_information_schema) {
+                $each_table['Rows'] = PMA_Table::countRecords($db,
+                    $each_table['Name'], $return = true);
+            }
+                
+            if ($is_show_stats) {
+                $tblsize                    =  doubleval($each_table['Data_length']) + doubleval($each_table['Index_length']);
+                $sum_size                   += $tblsize;
+                list($formatted_size, $unit) =  PMA_formatByteDown($tblsize, 3, ($tblsize > 0) ? 1 : 0);
+                if (isset($each_table['Data_free']) && $each_table['Data_free'] > 0) {
+                    list($formatted_overhead, $overhead_unit)     = PMA_formatByteDown($each_table['Data_free'], 3, ($each_table['Data_free'] > 0) ? 1 : 0);
+                    $overhead_size           += $each_table['Data_free'];
+                }
+            }
+            break;
+        case 'InnoDB' :
+            // InnoDB table: Row count is not accurate but data and index
+            // sizes are.
+
+            if ($each_table['TABLE_ROWS'] < $GLOBALS['cfg']['MaxExactCount']) {
+                $each_table['COUNTED'] = true;
+                $each_table['TABLE_ROWS'] = PMA_Table::countRecords($db,
+                    $each_table['TABLE_NAME'], $return = true, $force_exact = true,
+                    $is_view = false);
+            } else {
+                $each_table['COUNTED'] = false;
+            }
+
+            if ($is_show_stats) {
+                $tblsize                    =  $each_table['Data_length'] + $each_table['Index_length'];
+                $sum_size                   += $tblsize;
+                list($formatted_size, $unit) =  PMA_formatByteDown($tblsize, 3, ($tblsize > 0) ? 1 : 0);
+            }
+            //$display_rows                   =  ' - ';
+            break;
+        case 'MRG_MyISAM' :
+        case 'BerkeleyDB' :
+            // Merge or BerkleyDB table: Only row count is accurate.
+            if ($is_show_stats) {
+                $formatted_size =  ' - ';
+                $unit          =  '';
+            }
+            break;
+        // for a view, the ENGINE is null
+        case null :
+        case 'SYSTEM VIEW' :
+            // countRecords() takes care of $cfg['MaxExactCountViews']
+            $each_table['TABLE_ROWS'] = PMA_Table::countRecords($db,
+                    $each_table['TABLE_NAME'], $return = true, $force_exact = true,
+                    $is_view = true);
+            $table_is_view = true;
+            break;
+        default :
+            // Unknown table type.
+            if ($is_show_stats) {
+                $formatted_size =  'unknown';
+                $unit          =  '';
+            }
+    } // end switch
+    $sum_entries += $each_table['TABLE_ROWS'];
+
+    if (isset($each_table['Collation'])) {
+        $collation = '<dfn title="'
+            . PMA_getCollationDescr($each_table['Collation']) . '">'
+            . $each_table['Collation'] . '</dfn>';
+    } else {
+        $collation = '---';
+    }
+
+    if ($is_show_stats) {
+        if (isset($formatted_overhead)) {
+            $overhead = '<a href="tbl_structure.php?'
+                . $tbl_url_query . '#showusage">' . $formatted_overhead
+                . ' ' . $overhead_unit . '</a>' . "\n";
+            unset($formatted_overhead);
+            $overhead_check .=
+                "document.getElementById('checkbox_tbl_" . ($i + 1) . "').checked = true;";
+        } else {
+            $overhead = '-';
+        }
+    } // end if
 
     $alias = (!empty($tooltip_aliasname) && isset($tooltip_aliasname[$each_table['TABLE_NAME']]))
                ? str_replace(' ', '&nbsp;', htmlspecialchars($tooltip_truename[$each_table['TABLE_NAME']]))
@@ -209,13 +383,11 @@ foreach ($tables as $keyname => $each_table) {
                ? str_replace(' ', '&nbsp;', htmlspecialchars($tooltip_truename[$each_table['TABLE_NAME']]))
                : str_replace(' ', '&nbsp;', htmlspecialchars($each_table['TABLE_NAME']));
 
-    // Sets parameters for links
-    $tbl_url_query = $url_query . '&amp;table=' . $table_encoded;
     $i++;
 
     $row_count++;
     if ($table_is_view) {
-        $hidden_fields[] = '<input type="hidden" name="views[]" value="' .  $table_encoded . '" />';
+        $hidden_fields[] = '<input type="hidden" name="views[]" value="' .  htmlspecialchars($each_table['TABLE_NAME']) . '" />';
     }
 
     if ($each_table['TABLE_ROWS'] > 0) {
@@ -230,17 +402,10 @@ foreach ($tables as $keyname => $each_table) {
         if (! empty($each_table['TABLE_ROWS'])) {
             $empty_table = '<a href="sql.php?' . $tbl_url_query
                  . '&amp;sql_query=';
-            if (PMA_MYSQL_INT_VERSION >= 40000) {
-                $empty_table .= urlencode('TRUNCATE ' . PMA_backquote($each_table['TABLE_NAME']))
-                     . '&amp;zero_rows='
-                     . urlencode(sprintf($strTableHasBeenEmptied, htmlspecialchars($each_table['TABLE_NAME'])))
-                     . '" onclick="return confirmLink(this, \'TRUNCATE ';
-            } else {
-                $empty_table .= urlencode('DELETE FROM ' . PMA_backquote($each_table['TABLE_NAME']))
-                     . '&amp;zero_rows='
-                     . urlencode(sprintf($strTableHasBeenEmptied, htmlspecialchars($each_table['TABLE_NAME'])))
-                     . '" onclick="return confirmLink(this, \'DELETE FROM ';
-            }
+            $empty_table .= urlencode('TRUNCATE ' . PMA_backquote($each_table['TABLE_NAME']))
+                 . '&amp;zero_rows='
+                 . urlencode(sprintf($strTableHasBeenEmptied, htmlspecialchars($each_table['TABLE_NAME'])))
+                 . '" onclick="return confirmLink(this, \'TRUNCATE ';
             $empty_table .= PMA_jsFormat($each_table['TABLE_NAME']) . '\')">' . $titles['Empty'] . '</a>';
         } else {
             $empty_table = $titles['NoEmpty'];
@@ -253,77 +418,11 @@ foreach ($tables as $keyname => $each_table) {
             str_replace(' ', '&nbsp;', htmlspecialchars($each_table['TABLE_NAME'])));
     }
 
-    // loic1: Patch from Joshua Nye <josh at boxcarmedia.com> to get valid
-    //        statistics whatever is the table type
-
-    if (isset($each_table['TABLE_ROWS'])) {
-        // MyISAM, ISAM or Heap table: Row count, data size and index size
-        // is accurate.
-        if (preg_match('@^(MyISAM|ISAM|HEAP|MEMORY)$@', $each_table['ENGINE'])) {
-            if ($is_show_stats) {
-                $tblsize                    =  doubleval($each_table['Data_length']) + doubleval($each_table['Index_length']);
-                $sum_size                   += $tblsize;
-                list($formatted_size, $unit) =  PMA_formatByteDown($tblsize, 3, ($tblsize > 0) ? 1 : 0);
-                if (isset($each_table['Data_free']) && $each_table['Data_free'] > 0) {
-                    list($formatted_overhead, $overhead_unit)     = PMA_formatByteDown($each_table['Data_free'], 3, ($each_table['Data_free'] > 0) ? 1 : 0);
-                    $overhead_size           += $each_table['Data_free'];
-                }
-            }
-            $sum_entries                    += $each_table['TABLE_ROWS'];
-        } elseif ($each_table['ENGINE'] == 'InnoDB') {
-            // InnoDB table: Row count is not accurate but data and index
-            // sizes are.
-            if ($is_show_stats) {
-                $tblsize                    =  $each_table['Data_length'] + $each_table['Index_length'];
-                $sum_size                   += $tblsize;
-                list($formatted_size, $unit) =  PMA_formatByteDown($tblsize, 3, ($tblsize > 0) ? 1 : 0);
-            }
-            //$display_rows                   =  ' - ';
-            $sum_entries       += $each_table['TABLE_ROWS'];
-        } elseif (preg_match('@^(MRG_MyISAM|BerkeleyDB)$@', $each_table['ENGINE'])) {
-            // Merge or BerkleyDB table: Only row count is accurate.
-            if ($is_show_stats) {
-                $formatted_size =  ' - ';
-                $unit          =  '';
-            }
-            $sum_entries       += $each_table['TABLE_ROWS'];
-        } else {
-            // Unknown table type.
-            if ($is_show_stats) {
-                $formatted_size =  'unknown';
-                $unit          =  '';
-            }
-        }
-
-        if (PMA_MYSQL_INT_VERSION >= 40100) {
-            if (isset($each_table['Collation'])) {
-                $collation = '<dfn title="'
-                    . PMA_getCollationDescr($each_table['Collation']) . '">'
-                    . $each_table['Collation'] . '</dfn>';
-            } else {
-                $collation = '---';
-            }
-        }
-
-        if ($is_show_stats) {
-            if (isset($formatted_overhead)) {
-                $overhead = '<a href="tbl_structure.php?'
-                    . $tbl_url_query . '#showusage">' . $formatted_overhead
-                    . ' ' . $overhead_unit . '</a>' . "\n";
-                unset($formatted_overhead);
-                $overhead_check .=
-                    "document.getElementById('checkbox_tbl_$i').checked = true;";
-            } else {
-                $overhead = '-';
-            }
-        } // end if
-    } // end if (isset($each_table['TABLE_ROWS'])
-
     if ($num_columns > 0 && $num_tables > $num_columns
       && (($row_count % $num_columns) == 0)) {
         $row_count = 1;
         $odd_row = true;
-    ?>
+        ?>
     </tr>
 </tbody>
 </table>
@@ -334,7 +433,7 @@ foreach ($tables as $keyname => $each_table) {
 <tr class="<?php echo $odd_row ? 'odd' : 'even'; $odd_row = ! $odd_row; ?>">
     <td align="center">
         <input type="checkbox" name="selected_tbl[]"
-            value="<?php echo $table_encoded; ?>"
+            value="<?php echo htmlspecialchars($each_table['TABLE_NAME']); ?>"
             id="checkbox_tbl_<?php echo $i; ?>"<?php echo $checked; ?> /></td>
     <th><label for="checkbox_tbl_<?php echo $i; ?>"
             title="<?php echo $alias; ?>"><?php echo $truename; ?></label>
@@ -363,13 +462,13 @@ foreach ($tables as $keyname => $each_table) {
     // - when it's a view
     //  so ensure that we'll display "in use" below for a table
     //  that needs to be repaired
-
     if (isset($each_table['TABLE_ROWS']) && ($each_table['ENGINE'] != null || $table_is_view)) {
-        if ($table_is_view  && $each_table['TABLE_ROWS'] >= $cfg['MaxExactCountViews']) {
-            $at_least_one_view_exceeds_max_count = true;
-            $row_count_pre = '~';
-            $sum_row_count_pre = '~';
-            $show_superscript = '<sup>1</sup>';
+        if ($table_is_view) {
+            if ($each_table['TABLE_ROWS'] >= $GLOBALS['cfg']['MaxExactCountViews']){
+                $row_count_pre = '~';
+                $sum_row_count_pre = '~';
+                $show_superscript = PMA_showHint(PMA_sanitize(sprintf($strViewHasAtLeast, '[a@./Documentation.html#cfg_MaxExactCountViews@_blank]', '[/a]')));
+            }
         } elseif($each_table['ENGINE'] == 'InnoDB' && (! $each_table['COUNTED'])) {
             // InnoDB table: we did not get an accurate row count
             $row_count_pre = '~';
@@ -420,9 +519,16 @@ if ($is_show_stats) {
 ?>
 </tbody>
 <tbody>
-<tr><td></td>
+<tr><th></th>
     <th align="center" nowrap="nowrap">
-        <?php echo sprintf($strTables, PMA_formatNumber($num_tables, 0)); ?>
+        <?php
+            // for blobstreaming - if the number of tables is 0, set tableReductionCount to 0
+            // (we don't want negative numbers here) - rajk
+            if ($num_tables == 0)
+                $tableReductionCount = 0;
+
+            echo sprintf($strTables, PMA_formatNumber($num_tables - $tableReductionCount, 0));
+        ?>
     </th>
     <th colspan="<?php echo ($db_is_information_schema ? 3 : 6) ?>" align="center">
         <?php echo $strSum; ?></th>
@@ -433,17 +539,14 @@ if (!($cfg['PropertiesNumColumns'] > 1)) {
     echo '    <th align="center">' . "\n"
        . '        <dfn title="'
        . sprintf($strDefaultEngine, $default_engine) . '">' .$default_engine . '</dfn></th>' . "\n";
-    // Have to account for old MySQL with no collation (bug 1554885)
-    if (PMA_MYSQL_INT_VERSION >= 40100) {
-        // we got a case where $db_collation was empty
-        echo '    <th align="center">' . "\n";
-        if (! empty($db_collation)) {
-            echo '        <dfn title="'
-                . PMA_getCollationDescr($db_collation) . ' (' . $strDefault . ')">' . $db_collation
-                . '</dfn>';
-        }
-        echo '</th>';
+    // we got a case where $db_collation was empty
+    echo '    <th align="center">' . "\n";
+    if (! empty($db_collation)) {
+        echo '        <dfn title="'
+            . PMA_getCollationDescr($db_collation) . ' (' . $strDefault . ')">' . $db_collation
+            . '</dfn>';
     }
+    echo '</th>';
 }
 
 if ($is_show_stats) {
@@ -510,13 +613,6 @@ echo '    <option value="' . $strAnalyzeTable . '" >'
 </div>
 </form>
 <?php
-// Notice about row count for views
-
-if ($at_least_one_view_exceeds_max_count && !$db_is_information_schema) {
-    echo '<div class="notice">' . "\n";
-    echo '<sup>1</sup>' . PMA_sanitize(sprintf($strViewMaxExactCount, PMA_formatNumber($cfg['MaxExactCountViews'], 0), '[a@./Documentation.html#cfg_MaxExactCountViews@_blank]', '[/a]')) . "\n";
-    echo '</div>' . "\n";
-}
 // display again the table list navigator
 PMA_listNavigator($total_num_tables, $pos, $_url_params, 'db_structure.php', 'frame_content', $GLOBALS['cfg']['MaxTableList']);
 ?>
@@ -526,6 +622,11 @@ PMA_listNavigator($total_num_tables, $pos, $_url_params, 'db_structure.php', 'fr
 // Routines
 require './libraries/db_routines.inc.php';
 
+// Events
+if (PMA_MYSQL_INT_VERSION > 50100) {
+    require './libraries/db_events.inc.php';
+}
+
 /**
  * Work on the database
  * redesigned 2004-05-08 by mkkeck
