@@ -1,13 +1,17 @@
 <?php
-/* $Id: export.php 9054 2006-05-15 22:01:55Z nijel $ */
-// vim: expandtab sw=4 ts=4 sts=4:
+/* vim: set expandtab sw=4 ts=4 sts=4: */
+/**
+ * @todo    too much die here, or?
+ * @version $Id: export.php 12897 2009-08-30 12:43:07Z lem9 $
+ * @package phpMyAdmin
+ */
 
 /**
  * Get the variables sent or posted to this script and a core script
  */
-require_once('./libraries/common.lib.php');
-require_once('./libraries/zip.lib.php');
-require_once('./libraries/plugin_interface.lib.php');
+require_once './libraries/common.inc.php';
+require_once './libraries/zip.lib.php';
+require_once './libraries/plugin_interface.lib.php';
 
 PMA_checkParameters(array('what', 'export_type'));
 
@@ -22,21 +26,51 @@ if (!isset($export_list[$type])) {
     die('Bad type!');
 }
 
+/**
+ * valid compression methods
+ */
+$compression_methods = array(
+    'zip',
+    'gzip',
+    'bzip',
+);
+
+/**
+ * init and variable checking
+ */
+$compression = false;
+$onserver = false;
+$save_on_server = false;
+$buffer_needed = false;
+if (empty($_REQUEST['asfile'])) {
+    $asfile = false;
+} else {
+    $asfile = true;
+    if (in_array($_REQUEST['compression'], $compression_methods)) {
+        $compression = $_REQUEST['compression'];
+        $buffer_needed = true;
+    }
+    if (!empty($_REQUEST['onserver'])) {
+        $onserver = $_REQUEST['onserver'];
+        // Will we save dump on server?
+        $save_on_server = ! empty($cfg['SaveDir']) && $onserver;
+    }
+}
+
 // Does export require to be into file?
-if (isset($export_list[$type]['force_file']) && ! isset($asfile)) {
-    $message = $strExportMustBeFile;
-    $GLOBALS['show_error_header'] = true;
-    $js_to_run = 'functions.js';
-    require_once('./libraries/header.inc.php');
+if (isset($export_list[$type]['force_file']) && ! $asfile) {
+    $message = PMA_Message::error('strExportMustBeFile');
+    $GLOBALS['js_include'][] = 'functions.js';
+    require_once './libraries/header.inc.php';
     if ($export_type == 'server') {
         $active_page = 'server_export.php';
-        require('./server_export.php');
+        require './server_export.php';
     } elseif ($export_type == 'database') {
-        $active_page = 'db_details_export.php';
-        require('./db_details_export.php');
+        $active_page = 'db_export.php';
+        require './db_export.php';
     } else {
-        $active_page = 'tbl_properties_export.php';
-        require('./tbl_properties_export.php');
+        $active_page = 'tbl_export.php';
+        require './tbl_export.php';
     }
     exit();
 }
@@ -44,16 +78,22 @@ if (isset($export_list[$type]['force_file']) && ! isset($asfile)) {
 // Generate error url and check for needed variables
 if ($export_type == 'server') {
     $err_url = 'server_export.php?' . PMA_generate_common_url();
-} elseif ($export_type == 'database' && isset($db) && strlen($db)) {
-    $err_url = 'db_details_export.php?' . PMA_generate_common_url($db);
-} elseif ($export_type == 'table' && isset($db) && strlen($db) && isset($table) && strlen($table)) {
-    $err_url = 'tbl_properties_export.php?' . PMA_generate_common_url($db, $table);
+} elseif ($export_type == 'database' && strlen($db)) {
+    $err_url = 'db_export.php?' . PMA_generate_common_url($db);
+    // Check if we have something to export
+    if (isset($table_select)) {
+        $tables = $table_select;
+    } else {
+        $tables = array();
+    }
+} elseif ($export_type == 'table' && strlen($db) && strlen($table)) {
+    $err_url = 'tbl_export.php?' . PMA_generate_common_url($db, $table);
 } else {
     die('Bad parameters!');
 }
 
 // Get the functions specific to the export type
-require('./libraries/export/' . PMA_securePath($type) . '.php');
+require './libraries/export/' . PMA_securePath($type) . '.php';
 
 /**
  * Increase time limit for script execution and initializes some variables
@@ -111,9 +151,9 @@ function PMA_exportOutputHandler($line)
                 if ($GLOBALS['save_on_server']) {
                     $write_result = @fwrite($GLOBALS['file_handle'], $dump_buffer);
                     if (!$write_result || ($write_result != strlen($dump_buffer))) {
-                        $GLOBALS['message'] = sprintf($GLOBALS['strNoSpace'], htmlspecialchars($save_filename));
-                        $GLOBALS['show_error_header'] = TRUE;
-                        return FALSE;
+                        $GLOBALS['message'] = PMA_Message::error('strNoSpace');
+                        $GLOBALS['message']->addParam($save_filename);
+                        return false;
                     }
                 } else {
                     echo $dump_buffer;
@@ -130,12 +170,15 @@ function PMA_exportOutputHandler($line)
         }
     } else {
         if ($GLOBALS['asfile']) {
+            if ($GLOBALS['output_charset_conversion']) {
+                $line = PMA_convert_string($GLOBALS['charset'], $GLOBALS['charset_of_file'], $line);
+            }
             if ($GLOBALS['save_on_server'] && strlen($line) > 0) {
                 $write_result = @fwrite($GLOBALS['file_handle'], $line);
                 if (!$write_result || ($write_result != strlen($line))) {
-                    $GLOBALS['message'] = sprintf($GLOBALS['strNoSpace'], htmlspecialchars($save_filename));
-                    $GLOBALS['show_error_header'] = TRUE;
-                    return FALSE;
+                    $GLOBALS['message'] = PMA_Message::error('strNoSpace');
+                    $GLOBALS['message']->addParam($save_filename);
+                    return false;
                 }
                 $time_now = time();
                 if ($time_start >= $time_now + 30) {
@@ -144,9 +187,6 @@ function PMA_exportOutputHandler($line)
                 } // end if
             } else {
                 // We export as file - output normally
-                if ($GLOBALS['output_charset_conversion']) {
-                    $line = PMA_convert_string($GLOBALS['charset'], $GLOBALS['charset_of_file'], $line);
-                }
                 echo $line;
             }
         } else {
@@ -154,25 +194,9 @@ function PMA_exportOutputHandler($line)
             echo htmlspecialchars($line);
         }
     }
-    return TRUE;
+    return true;
 } // end of the 'PMA_exportOutputHandler()' function
 
-// Will we save dump on server?
-$save_on_server = isset($cfg['SaveDir']) && !empty($cfg['SaveDir']) && !empty($onserver);
-
-// Ensure compressed formats are associated with the download feature
-if (empty($asfile)) {
-    if ($save_on_server) {
-        $asfile = TRUE;
-    } elseif (isset($compression) && ($compression == 'zip' | $compression == 'gzip' | $compression == 'bzip')) {
-        $asfile = TRUE;
-    } else {
-        $asfile = FALSE;
-    }
-} else {
-    $asfile = TRUE;
-}
-
 // Defines the default <CR><LF> format. For SQL always use \n as MySQL wants this on all platforms.
 if ($what == 'sql') {
     $crlf = "\n";
@@ -183,16 +207,12 @@ if ($what == 'sql') {
 $output_kanji_conversion = function_exists('PMA_kanji_str_conv') && $type != 'xls';
 
 // Do we need to convert charset?
-$output_charset_conversion = $asfile &&
-    $cfg['AllowAnywhereRecoding'] && $allow_recoding
+$output_charset_conversion = $asfile && $cfg['AllowAnywhereRecoding']
     && isset($charset_of_file) && $charset_of_file != $charset
     && $type != 'xls';
 
-// Set whether we will need buffering
-$buffer_needed = isset($compression) && ($compression == 'zip' | $compression == 'gzip' | $compression == 'bzip');
-
-// Use on fly compression?
-$onfly_compression = $GLOBALS['cfg']['CompressOnFly'] && isset($compression) && ($compression == 'gzip' | $compression == 'bzip');
+// Use on the fly compression?
+$onfly_compression = $GLOBALS['cfg']['CompressOnFly'] && ($compression == 'gzip' || $compression == 'bzip');
 if ($onfly_compression) {
     $memory_limit = trim(@ini_get('memory_limit'));
     // 2 MB as default
@@ -226,23 +246,23 @@ if ($asfile) {
     $pma_uri_parts = parse_url($cfg['PmaAbsoluteUri']);
     if ($export_type == 'server') {
         if (isset($remember_template)) {
-            setcookie('pma_server_filename_template', $filename_template, 0, $GLOBALS['cookie_path'], '', $GLOBALS['is_https']);
+            PMA_setCookie('pma_server_filename_template', $filename_template);
         }
         $filename = str_replace('__SERVER__', $GLOBALS['cfg']['Server']['host'], strftime($filename_template));
     } elseif ($export_type == 'database') {
         if (isset($remember_template)) {
-            setcookie('pma_db_filename_template', $filename_template, 0, $GLOBALS['cookie_path'], '', $GLOBALS['is_https']);
+            PMA_setCookie('pma_db_filename_template', $filename_template);
         }
         $filename = str_replace('__DB__', $db, str_replace('__SERVER__', $GLOBALS['cfg']['Server']['host'], strftime($filename_template)));
     } else {
         if (isset($remember_template)) {
-            setcookie('pma_table_filename_template', $filename_template, 0, $GLOBALS['cookie_path'], '', $GLOBALS['is_https']);
+            PMA_setCookie('pma_table_filename_template', $filename_template);
         }
         $filename = str_replace('__TABLE__', $table, str_replace('__DB__', $db, str_replace('__SERVER__', $GLOBALS['cfg']['Server']['host'], strftime($filename_template))));
     }
 
     // convert filename to iso-8859-1, it is safer
-    if (!(isset($cfg['AllowAnywhereRecoding']) && $cfg['AllowAnywhereRecoding'] && $allow_recoding)) {
+    if (!(isset($cfg['AllowAnywhereRecoding']) && $cfg['AllowAnywhereRecoding'] )) {
         $filename = PMA_convert_string($charset, 'iso-8859-1', $filename);
     } else {
         $filename = PMA_convert_string($convcharset, 'iso-8859-1', $filename);
@@ -252,24 +272,15 @@ if ($asfile) {
     $filename  .= '.' . $export_list[$type]['extension'];
     $mime_type  = $export_list[$type]['mime_type'];
 
-    // If dump is going to be compressed, set correct encoding or mime_type and add
+    // If dump is going to be compressed, set correct mime_type and add
     // compression to extension
-    $content_encoding = '';
-    if (isset($compression) && $compression == 'bzip') {
+    if ($compression == 'bzip') {
         $filename  .= '.bz2';
-        // browsers don't like this:
-        //$content_encoding = 'x-bzip2';
         $mime_type = 'application/x-bzip2';
-    } elseif (isset($compression) && $compression == 'gzip') {
+    } elseif ($compression == 'gzip') {
         $filename  .= '.gz';
-        // Needed to avoid recompression by server modules like mod_gzip.
-        // It seems necessary to check about zlib.output_compression
-        // to avoid compressing twice
-        if (!@ini_get('zlib.output_compression')) {
-            $content_encoding = 'x-gzip';
-            $mime_type = 'application/x-gzip';
-        }
-    } elseif (isset($compression) && $compression == 'zip') {
+        $mime_type = 'application/x-gzip';
+    } elseif ($compression == 'zip') {
         $filename  .= '.zip';
         $mime_type = 'application/zip';
     }
@@ -280,31 +291,31 @@ if ($save_on_server) {
     $save_filename = PMA_userDir($cfg['SaveDir']) . preg_replace('@[/\\\\]@', '_', $filename);
     unset($message);
     if (file_exists($save_filename) && empty($onserverover)) {
-        $message = sprintf($strFileAlreadyExists, htmlspecialchars($save_filename));
-        $GLOBALS['show_error_header'] = TRUE;
+        $message = PMA_Message::error('strFileAlreadyExists');
+        $message->addParam($save_filename);
     } else {
         if (is_file($save_filename) && !is_writable($save_filename)) {
-            $message = sprintf($strNoPermission, htmlspecialchars($save_filename));
-            $GLOBALS['show_error_header'] = TRUE;
+            $message = PMA_Message::error('strNoPermission');
+            $message->addParam($save_filename);
         } else {
             if (!$file_handle = @fopen($save_filename, 'w')) {
-                $message = sprintf($strNoPermission, htmlspecialchars($save_filename));
-                $GLOBALS['show_error_header'] = TRUE;
+                $message = PMA_Message::error('strNoPermission');
+                $message->addParam($save_filename);
             }
         }
     }
     if (isset($message)) {
-        $js_to_run = 'functions.js';
-        require_once('./libraries/header.inc.php');
+        $GLOBALS['js_include'][] = 'functions.js';
+        require_once './libraries/header.inc.php';
         if ($export_type == 'server') {
             $active_page = 'server_export.php';
-            require('./server_export.php');
+            require './server_export.php';
         } elseif ($export_type == 'database') {
-            $active_page = 'db_details_export.php';
-            require('./db_details_export.php');
+            $active_page = 'db_export.php';
+            require './db_export.php';
         } else {
-            $active_page = 'tbl_properties_export.php';
-            require('./tbl_properties_export.php');
+            $active_page = 'tbl_export.php';
+            require './tbl_export.php';
         }
         exit();
     }
@@ -315,11 +326,12 @@ if ($save_on_server) {
  * or not
  */
 if (!$save_on_server) {
-    if ($asfile ) {
+    if ($asfile) {
         // Download
-        if (!empty($content_encoding)) {
-            header('Content-Encoding: ' . $content_encoding);
-        }
+        // (avoid rewriting data containing HTML with anchors and forms;
+        // this was reported to happen under Plesk)
+        @ini_set('url_rewriter.tags','');
+
         header('Content-Type: ' . $mime_type);
         header('Expires: ' . gmdate('D, d M Y H:i:s') . ' GMT');
         // lem9: Tested behavior of
@@ -334,11 +346,26 @@ if (!$save_on_server) {
             header('Pragma: public');
         } else {
             header('Pragma: no-cache');
+            // test case: exporting a database into a .gz file with Safari
+            // would produce files not having the current time
+            // (added this header for Safari but should not harm other browsers)
+            header('Last-Modified: ' . gmdate('D, d M Y H:i:s') . ' GMT');
         }
     } else {
         // HTML
+        if ($export_type == 'database') {
+            $num_tables = count($tables);
+            if ($num_tables == 0) {
+                $message = PMA_Message::error('strNoTablesFound');
+                $GLOBALS['js_include'][] = 'functions.js';
+                require_once './libraries/header.inc.php';
+                $active_page = 'db_export.php';
+                require './db_export.php';
+                exit();
+            }
+        }
         $backup_cfgServer = $cfg['Server'];
-        require_once('./libraries/header.inc.php');
+        require_once './libraries/header.inc.php';
         $cfg['Server'] = $backup_cfgServer;
         unset($backup_cfgServer);
         echo "\n" . '<div align="' . $cell_align_left . '">' . "\n";
@@ -352,28 +379,6 @@ if (!$save_on_server) {
     } // end download
 }
 
-// Check if we have something to export
-if ($export_type == 'database') {
-    $tables     = PMA_DBI_get_tables($db);
-    $num_tables = count($tables);
-    if ($num_tables == 0) {
-        $message = $strNoTablesFound;
-        $js_to_run = 'functions.js';
-        require_once('./libraries/header.inc.php');
-        if ($export_type == 'server') {
-            $active_page = 'server_export.php';
-            require('./server_export.php');
-        } elseif ($export_type == 'database') {
-            $active_page = 'db_details_export.php';
-            require('./db_details_export.php');
-        } else {
-            $active_page = 'tbl_properties_export.php';
-            require('./tbl_properties_export.php');
-        }
-        exit();
-    }
-}
-
 // Fake loop just to allow skip of remain of this code by break, I'd really
 // need exceptions here :-)
 do {
@@ -388,11 +393,11 @@ $do_relation = isset($GLOBALS[$what . '_relation']);
 $do_comments = isset($GLOBALS[$what . '_comments']);
 $do_mime     = isset($GLOBALS[$what . '_mime']);
 if ($do_relation || $do_comments || $do_mime) {
-    require_once('./libraries/relation.lib.php');
+    require_once './libraries/relation.lib.php';
     $cfgRelation = PMA_getRelationsParam();
 }
 if ($do_mime) {
-    require_once('./libraries/transformations.lib.php');
+    require_once './libraries/transformations.lib.php';
 }
 
 // Include dates in export?
@@ -403,19 +408,12 @@ $do_dates   = isset($GLOBALS[$what . '_dates']);
  */
 // Gets the number of tables if a dump of a database has been required
 if ($export_type == 'server') {
-    /**
-     * Gets the databases list - if it has not been built yet
-     */
-    if ($server > 0 && empty($dblist)) {
-        PMA_availableDatabases();
-    }
-
     if (isset($db_select)) {
         $tmp_select = implode($db_select, '|');
         $tmp_select = '|' . $tmp_select . '|';
     }
     // Walk over databases
-    foreach ($dblist AS $current_db) {
+    foreach ($GLOBALS['pma']->databases as $current_db) {
         if ((isset($tmp_select) && strpos(' ' . $tmp_select, '|' . $current_db . '|'))
             || !isset($tmp_select)) {
             if (!PMA_exportDBHeader($current_db)) {
@@ -429,26 +427,35 @@ if ($export_type == 'server') {
             foreach ($tables as $table) {
                 // if this is a view, collect it for later; views must be exported
                 // after the tables
-                if (PMA_Table::isView($current_db, $table)) {
+                $is_view = PMA_Table::isView($current_db, $table);
+                if ($is_view) {
                     $views[] = $table;
-                    continue;
                 }
-                $local_query  = 'SELECT * FROM ' . PMA_backquote($current_db) . '.' . PMA_backquote($table);
                 if (isset($GLOBALS[$what . '_structure'])) {
-                    if (!PMA_exportStructure($current_db, $table, $crlf, $err_url, $do_relation, $do_comments, $do_mime, $do_dates)) {
+                    // for a view, export a stand-in definition of the table
+                    // to resolve view dependencies
+                    if (!PMA_exportStructure($current_db, $table, $crlf, $err_url, $do_relation, $do_comments, $do_mime, $do_dates, $is_view ? 'stand_in' : 'create_table', $export_type)) {
                         break 3;
                     }
                 }
-                if (isset($GLOBALS[$what . '_data'])) {
+                if (isset($GLOBALS[$what . '_data']) && ! $is_view) {
+                    $local_query  = 'SELECT * FROM ' . PMA_backquote($current_db) . '.' . PMA_backquote($table);
                     if (!PMA_exportData($current_db, $table, $crlf, $err_url, $local_query)) {
                         break 3;
                     }
                 }
+                // now export the triggers (needs to be done after the data because
+                // triggers can modify already imported tables)
+                if (isset($GLOBALS[$what . '_structure'])) {
+                    if (!PMA_exportStructure($current_db, $table, $crlf, $err_url, $do_relation, $do_comments, $do_mime, $do_dates, 'triggers', $export_type)) {
+                        break 2;
+                    }
+                }
             }
             foreach($views as $view) {
                 // no data export for a view
                 if (isset($GLOBALS[$what . '_structure'])) {
-                    if (!PMA_exportStructure($current_db, $view, $crlf, $err_url, $do_relation, $do_comments, $do_mime, $do_dates)) {
+                    if (!PMA_exportStructure($current_db, $view, $crlf, $err_url, $do_relation, $do_comments, $do_mime, $do_dates, 'create_view', $export_type)) {
                         break 3;
                     }
                 }
@@ -462,40 +469,41 @@ if ($export_type == 'server') {
     if (!PMA_exportDBHeader($db)) {
         break;
     }
-
-    if (isset($table_select)) {
-        $tmp_select = implode($table_select, '|');
-        $tmp_select = '|' . $tmp_select . '|';
-    }
     $i = 0;
     $views = array();
+    // $tables contains the choices from the user (via $table_select)
     foreach ($tables as $table) {
         // if this is a view, collect it for later; views must be exported after
         // the tables
-        if (PMA_Table::isView($db, $table)) {
+        $is_view = PMA_Table::isView($db, $table);
+        if ($is_view) {
             $views[] = $table;
-            continue;
         }
-        $local_query  = 'SELECT * FROM ' . PMA_backquote($db) . '.' . PMA_backquote($table);
-        if ((isset($tmp_select) && strpos(' ' . $tmp_select, '|' . $table . '|'))
-            || !isset($tmp_select)) {
-
-            if (isset($GLOBALS[$what . '_structure'])) {
-                if (!PMA_exportStructure($db, $table, $crlf, $err_url, $do_relation, $do_comments, $do_mime, $do_dates)) {
-                    break 2;
-                }
+        if (isset($GLOBALS[$what . '_structure'])) {
+            // for a view, export a stand-in definition of the table
+            // to resolve view dependencies
+            if (!PMA_exportStructure($db, $table, $crlf, $err_url, $do_relation, $do_comments, $do_mime, $do_dates, $is_view ? 'stand_in' : 'create_table', $export_type)) {
+                break 2;
             }
-            if (isset($GLOBALS[$what . '_data'])) {
-                if (!PMA_exportData($db, $table, $crlf, $err_url, $local_query)) {
-                    break 2;
-                }
+        }
+        if (isset($GLOBALS[$what . '_data']) && ! $is_view) {
+            $local_query  = 'SELECT * FROM ' . PMA_backquote($db) . '.' . PMA_backquote($table);
+            if (!PMA_exportData($db, $table, $crlf, $err_url, $local_query)) {
+                break 2;
+            }
+        }
+        // now export the triggers (needs to be done after the data because
+        // triggers can modify already imported tables)
+        if (isset($GLOBALS[$what . '_structure'])) {
+            if (!PMA_exportStructure($db, $table, $crlf, $err_url, $do_relation, $do_comments, $do_mime, $do_dates, 'triggers', $export_type)) {
+                break 2;
             }
         }
     }
     foreach ($views as $view) {
         // no data export for a view
         if (isset($GLOBALS[$what . '_structure'])) {
-            if (!PMA_exportStructure($db, $view, $crlf, $err_url, $do_relation, $do_comments, $do_mime, $do_dates)) {
+            if (!PMA_exportStructure($db, $view, $crlf, $err_url, $do_relation, $do_comments, $do_mime, $do_dates, 'create_view', $export_type)) {
                 break 2;
             }
         }
@@ -518,30 +526,37 @@ if ($export_type == 'server') {
         $add_query  = '';
     }
 
-    if (!empty($sql_query)) {
-        // only preg_replace if needed
-        if (!empty($add_query)) {
-            // remove trailing semicolon before adding a LIMIT
-            $sql_query = preg_replace('%;\s*$%', '', $sql_query);
-        }
-        $local_query = $sql_query . $add_query;
-        PMA_DBI_select_db($db);
-    } else {
-        $local_query  = 'SELECT * FROM ' . PMA_backquote($db) . '.' . PMA_backquote($table) . $add_query;
-    }
-
+    $is_view = PMA_Table::isView($db, $table);
     if (isset($GLOBALS[$what . '_structure'])) {
-        if (!PMA_exportStructure($db, $table, $crlf, $err_url, $do_relation, $do_comments, $do_mime, $do_dates)) {
+        if (!PMA_exportStructure($db, $table, $crlf, $err_url, $do_relation, $do_comments, $do_mime, $do_dates, $is_view ? 'create_view' : 'create_table', $export_type)) {
             break;
         }
     }
-    // I think we have to export data for a single view; for example PDF report
-    //if (isset($GLOBALS[$what . '_data']) && ! PMA_table::isView($db, $table)) {
+    // If this is an export of a single view, we have to export data;
+    // for example, a PDF report
     if (isset($GLOBALS[$what . '_data'])) {
+        if (!empty($sql_query)) {
+            // only preg_replace if needed
+            if (!empty($add_query)) {
+                // remove trailing semicolon before adding a LIMIT
+                $sql_query = preg_replace('%;\s*$%', '', $sql_query);
+            }
+            $local_query = $sql_query . $add_query;
+            PMA_DBI_select_db($db);
+        } else {
+            $local_query  = 'SELECT * FROM ' . PMA_backquote($db) . '.' . PMA_backquote($table) . $add_query;
+        }
         if (!PMA_exportData($db, $table, $crlf, $err_url, $local_query)) {
             break;
         }
     }
+    // now export the triggers (needs to be done after the data because
+    // triggers can modify already imported tables)
+    if (isset($GLOBALS[$what . '_structure'])) {
+        if (!PMA_exportStructure($db, $table, $crlf, $err_url, $do_relation, $do_comments, $do_mime, $do_dates, 'triggers', $export_type)) {
+            break 2;
+        }
+    }
     if (!PMA_exportDBFooter($db)) {
         break;
     }
@@ -550,21 +565,21 @@ if (!PMA_exportFooter()) {
     break;
 }
 
-} while (FALSE);
+} while (false);
 // End of fake loop
 
 if ($save_on_server && isset($message)) {
-    $js_to_run = 'functions.js';
-    require_once('./libraries/header.inc.php');
+    $GLOBALS['js_include'][] = 'functions.js';
+    require_once './libraries/header.inc.php';
     if ($export_type == 'server') {
         $active_page = 'server_export.php';
-        require('./server_export.php');
+        require './server_export.php';
     } elseif ($export_type == 'database') {
-        $active_page = 'db_details_export.php';
-        require('./db_details_export.php');
+        $active_page = 'db_export.php';
+        require './db_export.php';
     } else {
-        $active_page = 'tbl_properties_export.php';
-        require('./tbl_properties_export.php');
+        $active_page = 'tbl_export.php';
+        require './tbl_export.php';
     }
     exit();
 }
@@ -579,8 +594,8 @@ if (!empty($asfile)) {
     }
 
     // Do the compression
-    // 1. as a gzipped file
-    if (isset($compression) && $compression == 'zip') {
+    // 1. as a zipped file
+    if ($compression == 'zip') {
         if (@function_exists('gzcompress')) {
             $zipfile = new zipfile();
             $zipfile -> addFile($dump_buffer, substr($filename, 0, -4));
@@ -588,19 +603,14 @@ if (!empty($asfile)) {
         }
     }
     // 2. as a bzipped file
-    elseif (isset($compression) && $compression == 'bzip') {
+    elseif ($compression == 'bzip') {
         if (@function_exists('bzcompress')) {
             $dump_buffer = bzcompress($dump_buffer);
-            if ($dump_buffer === -8) {
-                require_once('./libraries/header.inc.php');
-                echo sprintf($strBzError, '<a href="http://bugs.php.net/bug.php?id=17300" target="_blank">17300</a>');
-                require_once('./libraries/footer.inc.php');
-            }
         }
     }
     // 3. as a gzipped file
-    elseif (isset($compression) && $compression == 'gzip') {
-        if (@function_exists('gzencode')) {
+    elseif ($compression == 'gzip') {
+        if (@function_exists('gzencode') && !@ini_get('zlib.output_compression')) {
             // without the optional parameter level because it bug
             $dump_buffer = gzencode($dump_buffer);
         }
@@ -611,22 +621,22 @@ if (!empty($asfile)) {
         $write_result = @fwrite($file_handle, $dump_buffer);
         fclose($file_handle);
         if (strlen($dump_buffer) !=0 && (!$write_result || ($write_result != strlen($dump_buffer)))) {
-            $message = sprintf($strNoSpace, htmlspecialchars($save_filename));
+            $message = new PMA_Message('strNoSpace', PMA_Message::ERROR, $save_filename);
         } else {
-            $message = sprintf($strDumpSaved, htmlspecialchars($save_filename));
+            $message = new PMA_Message('strDumpSaved', PMA_Message::SUCCESS, $save_filename);
         }
 
-        $js_to_run = 'functions.js';
-        require_once('./libraries/header.inc.php');
+        $GLOBALS['js_include'][] = 'functions.js';
+        require_once './libraries/header.inc.php';
         if ($export_type == 'server') {
             $active_page = 'server_export.php';
-            require_once('./server_export.php');
+            require_once './server_export.php';
         } elseif ($export_type == 'database') {
-            $active_page = 'db_details_export.php';
-            require_once('./db_details_export.php');
+            $active_page = 'db_export.php';
+            require_once './db_export.php';
         } else {
-            $active_page = 'tbl_properties_export.php';
-            require_once('./tbl_properties_export.php');
+            $active_page = 'tbl_export.php';
+            require_once './tbl_export.php';
         }
         exit();
     } else {
@@ -646,7 +656,7 @@ else {
     echo '</div>' . "\n";
     echo "\n";
 ?>
-<script type="text/javascript" language="javascript">
+<script type="text/javascript">
 //<![CDATA[
     var bodyWidth=null; var bodyHeight=null;
     if (document.getElementById('textSQLDUMP')) {
@@ -667,6 +677,6 @@ else {
 //]]>
 </script>
 <?php
-    require_once('./libraries/footer.inc.php');
+    require_once './libraries/footer.inc.php';
 } // end if
 ?>
