@@ -3,12 +3,19 @@
 /**
  *
  *
- * @version $Id: Config.class.php 11669 2008-10-22 21:03:46Z lem9 $
+ * @version $Id: Config.class.php 13036 2009-10-12 21:59:48Z lem9 $
+ * @package phpMyAdmin
  */
 
+/**
+ * Load vendor configuration.
+ */
+require_once('./libraries/vendor_config.php');
+
 /**
  * Configuration class
  *
+ * @package phpMyAdmin
  */
 class PMA_Config
 {
@@ -85,7 +92,7 @@ class PMA_Config
      */
     function checkSystem()
     {
-        $this->set('PMA_VERSION', '3.0.1');
+        $this->set('PMA_VERSION', '3.2.2.1');
         /**
          * @deprecated
          */
@@ -168,7 +175,6 @@ class PMA_Config
         } elseif (preg_match('@OmniWeb/([0-9].[0-9]{1,2})@', $HTTP_USER_AGENT, $log_version)) {
             $this->set('PMA_USR_BROWSER_VER', $log_version[1]);
             $this->set('PMA_USR_BROWSER_AGENT', 'OMNIWEB');
-        //} elseif (ereg('Konqueror/([0-9].[0-9]{1,2})', $HTTP_USER_AGENT, $log_version)) {
         // Konqueror 2.2.2 says Konqueror/2.2.2
         // Konqueror 3.0.3 says Konqueror/3
         } elseif (preg_match('@(Konqueror/)(.*)(;)@', $HTTP_USER_AGENT, $log_version)) {
@@ -200,9 +206,6 @@ class PMA_Config
         } elseif ($this->get('GD2Available') == 'no') {
             $this->set('PMA_IS_GD2', 0);
         } else {
-            if (!@extension_loaded('gd')) {
-                PMA_dl('gd');
-            }
             if (!@function_exists('imagecreatetruecolor')) {
                 $this->set('PMA_IS_GD2', 0);
             } else {
@@ -214,7 +217,7 @@ class PMA_Config
                         $this->set('PMA_IS_GD2', 0);
                     }
                 } else {
-                    /* We must do hard way... */
+                    /* We must do hard way... but almost no chance to execute this */
                     ob_start();
                     phpinfo(INFO_MODULES); /* Only modules */
                     $a = strip_tags(ob_get_contents());
@@ -301,7 +304,8 @@ class PMA_Config
      */
     function __wakeup()
     {
-        if (! $this->checkConfigSource()
+        if (SKIP_MTIME_CONFIG_CHECK
+          || ! $this->checkConfigSource()
           || $this->source_mtime !== filemtime($this->getSource())
           || $this->default_source_mtime !== filemtime($this->default_source)
           || $this->error_config_file
@@ -405,6 +409,8 @@ class PMA_Config
         //$this->checkPmaAbsoluteUri();
         $this->settings = PMA_array_merge_recursive($this->settings, $cfg);
 
+        $this->checkPermissions();
+
         // Handling of the collation must be done after merging of $cfg
         // (from config.inc.php) so that $cfg['DefaultConnectionCollation']
         // can have an effect. Note that the presence of collation
@@ -479,8 +485,17 @@ class PMA_Config
             die('Existing configuration file (' . $this->getSource() . ') is not readable.');
         }
 
+        return true;
+    }
+
+    /**
+     * verifies the permissions on config file (if asked by configuration) 
+     * (must be called after config.inc.php has been merged)
+     */
+    function checkPermissions()
+    {
         // Check for permissions (on platforms that support it):
-	if ($this->get('CheckConfigurationPermissions')) {
+        if ($this->get('CheckConfigurationPermissions')) {
             $perms = @fileperms($this->getSource());
             if (!($perms === false) && ($perms & 2)) {
                 // This check is normally done after loading configuration
@@ -489,10 +504,8 @@ class PMA_Config
                     $this->source_mtime = 0;
                     die('Wrong permissions on configuration file, should not be world writable!');
                 }
-	    }
-	}
-
-        return true;
+            }
+        }
     }
 
     /**
@@ -541,7 +554,7 @@ class PMA_Config
      */
     function getThemeUniqueValue()
     {
-        return intval((null !== $_SESSION['PMA_Config']->get('fontsize') ? $_SESSION['PMA_Config']->get('fontsize') : $_COOKIE['pma_fontsize'])) + ($this->source_mtime + $this->default_source_mtime + $_SESSION['PMA_Theme']->mtime_info + $_SESSION['PMA_Theme']->filesize_info) . (isset($_SESSION['userconf']['custom_color']) ? substr($_SESSION['userconf']['custom_color'],1,6) : '');
+        return intval((null !== $_SESSION['PMA_Config']->get('fontsize') ? $_SESSION['PMA_Config']->get('fontsize') : (isset($_COOKIE['pma_fontsize']) ? $_COOKIE['pma_fontsize'] : 0))) + ($this->source_mtime + $this->default_source_mtime + $_SESSION['PMA_Theme']->mtime_info + $_SESSION['PMA_Theme']->filesize_info) . (isset($_SESSION['userconf']['custom_color']) ? substr($_SESSION['userconf']['custom_color'],1,6) : '');
     }
 
     /**
@@ -590,9 +603,10 @@ class PMA_Config
 
                 // Host and port
                 if (PMA_getenv('HTTP_HOST')) {
-                    if (strpos(PMA_getenv('HTTP_HOST'), ':') !== false) {
-                        list($url['host'], $url['port']) =
-                            explode(':', PMA_getenv('HTTP_HOST'));
+                    // Prepend the scheme before using parse_url() since this is not part of the RFC2616 Host request-header
+                    $parsed_url = parse_url($url['scheme'] . '://' . PMA_getenv('HTTP_HOST'));
+                    if (!empty($parsed_url['host'])) {
+                        $url = $parsed_url;
                     } else {
                         $url['host'] = PMA_getenv('HTTP_HOST');
                     }
